<?php

namespace Drupal\panopto_media_remote\Plugin\Field\FieldFormatter;

use Drupal\Component\Utility\Html;
use Drupal\Core\Field\Attribute\FieldFormatter;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\media_remote\Plugin\Field\FieldFormatter\MediaRemoteFormatterBase;

/**
 * Plugin implementation of the 'panopto' formatter.
 */
#[FieldFormatter(
  id: "panopto",
  label: new TranslatableMarkup("Remote Media - Panopto"),
  field_types: ["string"]
)]
class PanoptoFormatter extends MediaRemoteFormatterBase {

  /**
   * {@inheritdoc}
   */
  public static function getUrlRegexPattern(): string {
    return '/^https:\/\/(?:[a-zA-Z0-9-]+\.)+[a-zA-Z0-9-]+\.panopto\.[a-zA-Z]+\/Panopto\/Pages\/(?:Embed|Viewer)(?:\.aspx)?\?id=([a-zA-Z0-9-]+)/';
  }

  /**
   * {@inheritdoc}
   */
  public static function getValidUrlExampleStrings(): array {
    return [
      'https://hosted.cloud.panopto.eu/Panopto/Pages/Embed.aspx?id=[id]',
      'https://hosted.cloud.panopto.eu/Panopto/Pages/Viewer.aspx?id=[id]',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public static function deriveMediaDefaultNameFromUrl($url): TranslatableMarkup {
    $pattern = static::getUrlRegexPattern();
    if (preg_match($pattern, $url)) {
      return t('Panopto from @url', [
        '@url' => self::getEmbedUrl($url),
      ]);
    }
    return parent::deriveMediaDefaultNameFromUrl($url);
  }

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode): array {
    $elements = [];
    foreach ($items as $delta => $item) {
      if ($item->isEmpty()) {
        continue;
      }
      $elements[$delta] = [
        '#theme' => 'panopto',
        '#title' => $item->getEntity()->label() ?? $this->t('Panopto embed'),
        '#url' => self::getEmbedUrl($item->value),
        '#width' => Html::escape($this->getSetting('width') ?? '640px'),
        '#height' => Html::escape($this->getSetting('height') ?? '480px'),
      ];
    }
    return $elements;
  }

  /**
   * Converts the user provided url into a valid embedding url.
   *
   * @param string $url
   *   Initial url.
   *
   * @return string
   *   A url valid for embedding.
   */
  protected static function getEmbedUrl(string $url): string {
    return $url;
  }

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings(): array {
    return [
        'width' => '640px',
        'height' => '480px',
      ] + parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state): array {
    return parent::settingsForm($form, $form_state) + [
        'width' => [
          '#type' => 'textfield',
          '#title' => $this->t('Width'),
          '#default_value' => $this->getSetting('width'),
          '#size' => 10,
          '#maxlength' => 10,
          '#description' => $this->t('Iframe width, in pixels or as a percentage of the available area. Examples: 640px, 100%.'),
        ],
        'height' => [
          '#type' => 'textfield',
          '#title' => $this->t('Height'),
          '#default_value' => $this->getSetting('height'),
          '#size' => 10,
          '#maxlength' => 10,
          '#description' => $this->t('Iframe height, in pixels or as a percentage of the available area. Examples: 480px, 100%.'),
        ],
      ];
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary(): array {
    $summary = parent::settingsSummary();
    $summary[] = $this->t('Iframe size: %width x %height', [
      '%width' => $this->getSetting('width'),
      '%height' => $this->getSetting('height'),
    ]);
    return $summary;
  }

}
